import fs from "fs";
import express from "express";

const router = express.Router();
const videosDirectory = process.env.VIDEOS_DIR || "./videos";
const spritesDirectory = process.env.SPRITES_DIR || "./sprites";

import {
  createDirectory,
  downloadVideo,
  extractFrames,
  SpriteResult,
} from "../utils/utils";
import path from "path";

router.get("/", async (req: any, res: any, next: any) => {
  // return list of sprites
  try {
    const spritesDir = path.resolve(__dirname, "../..", spritesDirectory);
    fs.readdir(spritesDir, (err, files) => {
      if (err) {
        res.json({ sprites: [] });
      } else {
        res.json({
          sprites: files
            .filter((f) => !f.startsWith("."))
            .map((file) => {
              try {
                const spritesPath = `${spritesDir}/${file}`;
                const metadataPath = `${spritesPath}/video-${file}.json`;
                fs.accessSync(metadataPath);
                const json = fs.readFileSync(metadataPath).toString();
                const sprites = fs
                  .readdirSync(spritesPath)
                  .filter((f) => f.endsWith(".jpeg"));
                return {
                  ...JSON.parse(json),
                  sprites,
                };
              } catch (err) {
                return {
                  id: file,
                  error: "No metadata",
                };
              }
            }),
        });
      }
    });
  } catch (err) {
    next(err);
  }
});

router.get("/:id", async (req: any, res: any, next: any) => {
  // return requested sprites paths
  try {
    const videoId = req.params.id;
    const spritesDir = path.resolve(
      __dirname,
      "../..",
      spritesDirectory,
      videoId
    );
    fs.readdir(spritesDir, (err, files) => {
      if (err)
        res.json({
          error: "Error reading sprites, try generate sprites first.",
        });
      if (files) {
        res.json({
          metadata: files
            .filter((f) => f.endsWith(".json"))
            .map((f) =>
              JSON.parse(fs.readFileSync(`${spritesDir}/${f}`).toString())
            )
            .shift(),
          sprites: files
            .filter((f) => f.endsWith(".jpeg"))
            .map((f) => `/public/${videoId}/${f}`),
        });
      }
    });
  } catch (err) {
    next(err);
  }
});

router.put("/:id", async (req: any, res: any, next: any) => {
  // update submitted sprite
  try {
    const id = req.params.id;
    const { src } = req.body;
    if (!src) res.json({ error: "Missing required json body property [src]" });

    if (!src.startsWith("http"))
      res.json({ error: "Property [src] should be valid url." });

    createDirectory(videosDirectory, () => {
      downloadVideo(src, id).then((path) => {
        extractFrames(src, path, id).then((result: SpriteResult) => {
          res.json({
            id: id,
            thumbWidth: result.thumbWidth,
            thumbHeight: result.thumbHeight,
            sprites: result.sprites,
          });
        });
      });
    });
  } catch (err) {
    next(err);
  }
});

router.delete("/:id", async (req: any, res: any, next: any) => {
  // trash sprite
  try {
    res.json({ error: "Not implemented" });
  } catch (err) {
    next(err);
  }
});

module.exports = router;
