import express from "express";
import path from "path";
import cors from "cors";
import * as dotenv from "dotenv";

dotenv.config();

const PORT = process.env.PORT ? +process.env.PORT : 3500;
const spritesDirectory = process.env.SPRITES_DIR || "./sprites";
const allowedOrigins = process.env.ALLOWED_ORIGIN?.split(",") || [
  "http://localhost:3000",
];
const app = express();

const corsOptions: cors.CorsOptions = {
  allowedHeaders: [
    "Origin",
    "X-Requested-With",
    "Content-Type",
    "Accept",
    "X-Access-Token",
  ],
  credentials: true,
  methods: "GET,HEAD,OPTIONS,PUT,PATCH,POST,DELETE",
  origin: (origin, callback) => {
    if (origin && allowedOrigins.indexOf(origin) !== -1) {
      callback(null, true);
    } else {
      callback(new Error("Not allowed by CORS"));
    }
  },
  preflightContinue: false,
};

app.listen(PORT, () => console.log(`[Express] Listening on port ${PORT}`));
app.use(cors(corsOptions));
app.use(express.json());
app.use("/", require("./api"));
app.use("*", express.static(path.resolve(__dirname, "..", "build")));
app.use(
  "/public",
  express.static(path.resolve(__dirname, "..", spritesDirectory))
);

// error handling endware
app.use((err: any, req: any, res: any, next: any) => {
  console.error(err);
  res.status(err.status || 500).send(err.message || "Internal server error.");
  next();
});
