import { downloadVideo, extractFrames } from "./utils/utils";
import fs from "fs";
import axios from "axios";

const hydraUrl = `https://hydra.joystream.org/graphql`;
const providerUrl = `https://ru.joystreamstats.live/storage/asset/v0/`;

type Task = () => void;
const queue: Task[] = [];
const maxWorkers = 16;
let workers = 0;

interface Video {
  id: number;
  mediaDataObject: { joystreamContentId: string };
}

const fetchVideos = async (): Promise<Video[]> => {
  const db = `../videos.json`;
  if (fs.existsSync(db)) return JSON.parse(fs.readFileSync(db, `utf-8`));

  const query = {
    query: `query {\n videos (limit:5000) {\n id,\n mediaDataObject {\n joystreamContentId,\n },\n  }}\n`,
  };
  const { data } = await axios.post(hydraUrl, query);
  const videos: Video[] = data.data.videos;
  fs.writeFileSync(db, JSON.stringify(videos) + `\n\n`);
  return videos;
};

const createSprites = async (
  url: string,
  file: string,
  id: number
): Promise<void> => {
  await extractFrames(url, file, id);
  fs.rm(file, (err) => console.error(err));
};

const processVideo = async (video: Video): Promise<void> => {
  const { id, mediaDataObject } = video;
  const file = `../videos/${id}.mp4`;

  if (fs.existsSync(`../spites/${id}/video-${id}.json`))
    return console.log(`[${id}] Skipping (sprites exist)`);

  const url = providerUrl + mediaDataObject.joystreamContentId;
  if (fs.existsSync(`../videos/${id}.mp4`)) return createSprites(url, file, id);

  try {
    const path: string = await downloadVideo(url, String(id));
    return createSprites(url, path, id);
  } catch (err) {
    fs.appendFileSync(`error.log`, JSON.stringify(err));
  }
};

const tick = async (): Promise<void> => {
  if (maxWorkers > workers + 1) setTimeout(tick, 0);

  const task = queue.pop();
  if (!task) return;

  workers++;
  const result = await task();
  workers--;

  setTimeout(tick, 0);
};

const main = async (): Promise<void> => {
  const videos: Video[] = await fetchVideos();
  for (let i = 0; i < videos.length; i++)
    queue.push(() => processVideo(videos[i]));
  tick();
};
main();
